from enum import Enum


class Language(Enum):
    ALBANIAN = "sq"
    ARABIC = "ar"
    CATALAN = "ca"
    DUTCH = "nl"
    BOSNIAN = "bs"
    BULGARIAN = "bg"
    CROATIAN = "hr"
    CZECH = "cs"
    DANISH = "da"
    ESTONIAN = "et"
    FINNISH = "fi"
    FRENCH = "fr"
    GERMAN = "de"
    GREEK = "el"
    HUNGARIAN = "hu"
    ICELANDIC = "is"
    IRISH = "ga"
    HEBREW = "he"
    ITALIAN = "it"
    SERBIAN = "sr"
    LITHUANIAN = "lt"
    MALTESE = "mt"
    MACEDONIAN = "mk"
    NORWEGIAN = "no"
    POLISH = "pl"
    PORTUGUESE = "pt"
    ROMANIAN = "ro"
    SLOVAK = "sk"
    SLOVENIAN = "sl"
    ENGLISH = "en"
    SPANISH = "es"
    SWEDISH = "sv"
    TURKISH = "tr"
